module Honey
  #
  # Data Feed
  #
  # Access Honey's Place data feeds in various formats
  #
  class DataFeed
    attr_reader :code, :format

    Formats = %i(xml json csv woo shopify amazon_inventory).freeze

    def initialize(code, format)
      @code = code
      @format = format if Formats.include?(format)
    end

    def url
      if format.nil?
        raise ArgumentError, "format is required to be one of #{Formats}"
      end
      "https://www.honeysplace.com/df/#{code}/#{format}"
    end

    def parse
      read
    end

    def download_to(path)
      File.open(path, 'w') do |file|
        file.write(read)
        # HTTParty.get(url, stream: true) do |fragment|
        #   file.write(fragment)
        # end
      end
    end

    private

    def read
      response = HTTParty.get(url)
      if response.headers['content-type'].start_with?('text/html;')
        raise StandardError, 'This data feed is not ready. Try again later.'
      end
      response.parsed_response
    end
  end
end
