module Honey
  class Order
    def self.attr_accessor(*attributes)
      @attributes ||= []
      @attributes.concat attributes
      super(*attributes)
    end

    def self.attributes
      @attributes
    end

    def attributes
      self.class.attributes
    end

    attr_accessor :reference, :shipby, :items, :last, :first, :address1, :address2, :city, :state, :zip, :country, :phone, :emailaddress, :instructions, :date

    def initialize(args = {})
      update(args)
    end

    def update(args = {})
      args.each_pair do |key, value|
        self.send("#{key}=".to_sym, value)
      end
    end

    def reference=(id)
      if id == :test
        @reference = "TEST" << Digest::MD5.base64digest(Time.now.to_s).gsub(/\W/, '')
      else
        @reference = id
      end
    end

    def shipby=(code)
      if code.is_a?(String)
        @shipby = code
      else
        @shipby = shipping_option(code)
      end
    end
    alias_method :shipping, :shipby
    alias_method :shipping=, :shipby=

    def each_pair(&block)
      present_attributes.each do |key|
        yield(key, self.send(key), block)
      end
    end

    def valid?
      (required_attributes - present_attributes).empty?
    end

    def invalid?
      !valid?
    end

    def required_attributes
      attributes - [:address2, :instructions, :phone]
    end

    def present_attributes
      attributes.collect do |attr|
        attr unless self.send(attr).nil?
      end.compact
    end

    def self.shipping_option(carrier: :special, service:)
      carrier = carrier.to_s
      service = service.to_s
      unless carrier_options = shipping_options[carrier.downcase]
        raise Honey::Error, "'#{carrier}' isn't a valid carrier. Valid carriers are: #{shipping_options.keys}"
      end
      unless option = carrier_options[service.downcase]
        raise Honey::Error, "'#{service}' isn't a shipping option for #{carrier}. Valid options are: #{carrier_options.keys}"
      end
      option
    end

    def shipping_option(*args)
      self.class.shipping_option(*args)
    end

    def self.shipping_option_from_api_code(api_code)
      shipping_options.each_pair do |carrier, options|
        options.each_pair do |service, code|
          return { carrier: carrier, service: service } if api_code == code
        end
      end
      nil
    end

    private
    def self.shipping_options
      {
        "special" => {
          "pickup" => "PICKUP",
          "best rate" => "RTSHOP",
          "cheapest" => "RTSHOP"
        },
        "fedex" => {
          "first overnight" => "F001",
          "priority overnight" => "F002",
          "standard overnight" => "F003",
          "2 day air" => "F004",
          "express saver" => "F005",
          "ground north america" => "F006",
          "ground home delivery" => "F007",
          "international priority" => "F008",
          "international economy" => "F009"
        },
        "usps" => {
          "express mail" => "P001",
          "priority mail" => "P002",
          "first class" => "P003",
          "parcel post" => "P004",
          "international express" => "P005",
          "international priority" => "P006",
          "international first class" => "P007",
          "priority flat rate" => "P008"
        },
        "ups" => {
          "next day air" => "U001",
          "2 day air" => "U002",
          "3 day air" => "U003",
          "ground" => "U004",
          "standard" => "U005"
        }
      }
    end
  end
end
