require 'ostruct'

module Honey
  class Response
    attr_accessor :data

    def initialize(http_response)
      data = http_response.parsed_response['HPEnvelope']
      @data = OpenStruct.new(data)
    end

    def method_missing(method, *args, &block)
      if method.to_s.end_with?('=')
        data.send(method, *args)
      else
        data.send(method)
      end
    end

    def inspect
      data.inspect
    end

    def successful?
      !failed?
    end

    def failed?
      data.code.present? && data.code.to_i != 100
    end

    def cancelled?
      data.cancelled == 'Yes'
    end

    def shipped?
      data.status == 'Shipped'
    end

    def error
      Honey::Error.new(data.code.to_i)
    end

    def log
      "[#{data.code}] #{message}"
    end

    def message
      data.error || data.details
    end

    def tracking_numbers
      (1..5).map { |i| data.try(:"trackingnumber#{i}") }.compact.join(',')
    end

    def notes
      data.warehousenotes || ""
    end
  end
end
